//============================================
//    Test   SL025x
//MSP430 C source code
//MSP430F2232+IAR
//===========================================
#include <msp430x22x2.h>
#include <string.h>

//============================================
//  PIN define
//============================================
#define BuzzerON (P4OUT |=BIT2)
#define BuzzerOFF (P4OUT &= ~BIT2)
#define CARDIN (P1IN&BIT6)

//============================================
//  Uart status words define
//============================================
#define UARTSTATUS_FREE 0
#define UARTSTATUS_TX 1
#define UARTSTATUS_TXSUCC 2
#define UARTSTATUS_RXSUCC 3
#define UARTSTATUS_RXERR 4

//============================================
//  Card type define
//============================================
#define CARDTYPE_S50 0
#define CARDTYPE_S70 1
#define CARDTYPE_ProX 2
#define CARDTYPE_Pro 3
#define CARDTYPE_UL 4
#define CARDTYPE_DES 5
#define CARDTYPE_TAG 6
#define CARDTYPE_ICODE 7

//============================================
//  global variable define
//============================================
unsigned char g_cTimer_Buzzer;
unsigned char g_cCardType;
unsigned long int g_iTimer;
unsigned char g_bOverTime;
unsigned char g_cModel;
unsigned char g_cRxBuf[40];
unsigned long int lPurseValue;
unsigned char g_cUartTxCnt;
unsigned char g_cUartTxDataLen;
unsigned char *g_pUartTxDat;
unsigned char g_cUartTxCheckSum;
unsigned char g_cUartRxCnt;
unsigned char g_cUartStatus;

//============================================
//  Command List, preamble + length + command 
//============================================
const unsigned char UCMD_SelectCard[]             ={0xBA,0x02,0x01 };       
const unsigned char UCMD_LoginSector0[]           ={0xBA,0x0A,0x02,0x00,0xAA,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};  
const unsigned char UCMD_LoginSector1[]           ={0xBA,0x0A,0x02,0x01,0xAA,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
const unsigned char UCMD_ReadBlock1[]             ={0xBA,0x03,0x03,0x01};                                     
const unsigned char UCMD_WriteBlock1[]            ={0xBA,0x13,0x04,0x01,0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,0xAA,0xBB,0xCC,0xDD,0xEE,0xFF};                                
const unsigned char UCMD_ReadValue[]              ={0xBA,0x03,0x05,0x04};  
const unsigned char UCMD_WriteValue[]             ={0xBA,0x07,0x06,0x04,0x00,0x00,0x00,0x01};                                
const unsigned char UCMD_IncrementValue[]         ={0xBA,0x07,0x08,0x04,0x00,0x00,0x00,0x20};                                
const unsigned char UCMD_DecrementValue[]         ={0xBA,0x07,0x09,0x04,0x03,0x00,0x00,0x00};                                          
const unsigned char UCMD_CopyValue[]              ={0xBA,0x04,0x0A,0x04,0x05};
const unsigned char UCMD_ReadULPage5[]            ={0xBA,0x03,0x10,0x05};                                          
const unsigned char UCMD_WriteULPage5[]           ={0xBA,0x07,0x11,0x05,0x11,0x22,0x33,0x44}; 
const unsigned char UCMD_RATSDesFire[]            ={0xBA,0x02,0x20};
const unsigned char UCMD_DesFireGetVersion[]      ={0xBA,0x03,0x21,0x60}; 
const unsigned char UCMD_GetAdditionalData[]      ={0xBA,0x03,0x21,0xAF}; 
const unsigned char UCMD_LightOn[]                ={0xBA,0x03,0x40,0x01}; 
const unsigned char UCMD_LightOff[]               ={0xBA,0x03,0x40,0x00};       
const unsigned char UCMD_Halt[]                   ={0xBA,0x03,0x50,0x00};             
const unsigned char UCMD_GetCardInformation[]     ={0xBA,0x02,0x31};       
const unsigned char UCMD_GetBlcok0SecurityStatus[]={0xBA,0x04,0x32,0x0,0x01};
const unsigned char UCMD_ReadBlocks[]             ={0xBA,0x04,0x33,0x0,0x01};                                     
const unsigned char UCMD_WriteBlock0[]            ={0xBA,0x07,0x34,0x0,0x11,0x22,0x33,0x44};                                
const unsigned char UCMD_WriteAFI[]               ={0xBA,0x03,0x35,0xAA};  
const unsigned char UCMD_WriteDSFID[]             ={0xBA,0x03,0x36,0xBB};                                
const unsigned char UCMD_LockBlock0[]             ={0xBA,0x03,0x37,0x0};                                
const unsigned char UCMD_LockAFI[]                ={0xBA,0x02,0x38};                                          
const unsigned char UCMD_LockDSFID[]              ={0xBA,0x02,0x39};    

//============================================
//  procedure define
//============================================
void Init_Hardware(void);
void BuzzerOn(void);
void Start_Time(unsigned int ms);
void SendBuf_UART(const unsigned char *dat,unsigned char len);
void Test_SL025x(void);

//============================================
//  Timer interrupt service
//============================================
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A(void)
{
  if(g_cTimer_Buzzer!=0)
  {
    g_cTimer_Buzzer--;
    if(g_cTimer_Buzzer==0)  BuzzerOFF;
  }
  if(g_iTimer!=0)
  {
    g_iTimer--;
    if(g_iTimer==0) g_bOverTime=1;
  }
}

//============================================
//  Uart RX interrupt service
//============================================
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCI0RX_ISR(void)
{
  unsigned char dat;
  unsigned char CheckSum;
  unsigned char counter;
  dat=UCA0RXBUF;
  g_cRxBuf[g_cUartRxCnt]=dat;
  g_cUartRxCnt++;
  if(g_cUartRxCnt==g_cRxBuf[1]+2)
  {
    CheckSum=0;
    for(counter=0;counter<g_cUartRxCnt;counter++) CheckSum=CheckSum^g_cRxBuf[counter];
    if(CheckSum==0) g_cUartStatus=UARTSTATUS_RXSUCC;
    else g_cUartStatus=UARTSTATUS_RXERR;
  }
}

//============================================
//  initialize system hardware config
//  timer,uart,port
//============================================
void Init_Hardware(void)
{
  WDTCTL=WDTPW+WDTHOLD;
  BCSCTL1=CALBC1_1MHZ;
  DCOCTL=CALDCO_1MHZ;
  P3SEL=BIT5|BIT4|BIT2|BIT1;
  UCA0BR0=0x08;
  UCA0BR1=0;
  UCA0CTL1 |= UCSSEL_2;
  UCA0MCTL=UCBRS2+UCBRS0;
  UCA0CTL1 &= ~UCSWRST;
  IE2 |= UCA0RXIE;
  // Timer Initialize
  TACCTL0=CCIE;
  TACCR0=10000;
  TACTL=TASSEL_2+MC_1;
  // Port Initialize
  P3DIR |= 0x01;
  P4DIR |= 0x07;
  BuzzerOFF;
  __bis_SR_register(GIE);
}

//============================================
//  trun on buzzer about 100mS
//============================================
void BuzzerOn(void)
{
  BuzzerON;
  g_cTimer_Buzzer=10;
}

//============================================
//  start timer
//============================================
void Start_Time(unsigned int ms)
{
  g_iTimer=ms;
  g_bOverTime=0;
}

//============================================
//  send buffer by uart
//============================================
void SendBuf_UART(const unsigned char *dat,unsigned char len)
{
  unsigned char tmp;
  g_cUartStatus=UARTSTATUS_TX;
  g_pUartTxDat=(unsigned char*)(dat);
  g_cUartTxDataLen=len;
  g_cUartTxCnt=0;
  g_cUartTxCheckSum=0;
  while(g_cUartTxCnt!=g_cUartTxDataLen)
  {
    tmp=g_pUartTxDat[g_cUartTxCnt];
    g_cUartTxCheckSum=g_cUartTxCheckSum^tmp;
    g_cUartTxCnt++;
    while (!(IFG2&UCA0TXIFG));
    UCA0TXBUF=tmp;
  }
  g_cUartStatus=UARTSTATUS_TXSUCC;
  g_cUartRxCnt=0;
  while (!(IFG2&UCA0TXIFG));
  UCA0TXBUF=g_cUartTxCheckSum;
}

unsigned char Uart_Send(const unsigned char *inbuf,unsigned char inbufsize)
{
  SendBuf_UART(inbuf,inbufsize);	
  Start_Time(30);
  while((g_cUartStatus!=UARTSTATUS_RXERR)&&(g_cUartStatus!=UARTSTATUS_RXSUCC)&&(g_bOverTime==0));	
  if(g_cUartStatus!=UARTSTATUS_RXSUCC) return 0;
  return 1;
}

void Uart_Halt()
{
  SendBuf_UART((UCMD_Halt),sizeof(UCMD_Halt));	
  Start_Time(5);
  while(g_bOverTime==0);
}

unsigned char Uart_SelectCard() {return(Uart_Send(UCMD_SelectCard,sizeof(UCMD_SelectCard)));}
unsigned char Uart_LightOn() {return(Uart_Send(UCMD_LightOn,sizeof(UCMD_LightOn)));}
unsigned char Uart_LoginSector0() {return(Uart_Send(UCMD_LoginSector0,sizeof(UCMD_LoginSector0)));}
unsigned char Uart_WriteBlock1() {return(Uart_Send(UCMD_WriteBlock1,sizeof(UCMD_WriteBlock1)));}
unsigned char Uart_ReadBlock1() {return(Uart_Send(UCMD_ReadBlock1,sizeof(UCMD_ReadBlock1)));}
unsigned char Uart_LoginSector1() {return(Uart_Send(UCMD_LoginSector1,sizeof(UCMD_LoginSector1)));}
unsigned char Uart_WriteValue() {return(Uart_Send(UCMD_WriteValue,sizeof(UCMD_WriteValue)));}
unsigned char Uart_IncrementValue() {return(Uart_Send(UCMD_IncrementValue,sizeof(UCMD_IncrementValue)));}
unsigned char Uart_DecrementValue() {return(Uart_Send(UCMD_DecrementValue,sizeof(UCMD_DecrementValue)));}
unsigned char Uart_CopyValue() {return(Uart_Send(UCMD_CopyValue,sizeof(UCMD_CopyValue)));}
unsigned char Uart_ReadValue() {return(Uart_Send(UCMD_ReadValue,sizeof(UCMD_ReadValue)));}
unsigned char Uart_WriteULPage5() {return(Uart_Send(UCMD_WriteULPage5,sizeof(UCMD_WriteULPage5)));}
unsigned char Uart_ReadULPage5() {return(Uart_Send(UCMD_ReadULPage5,sizeof(UCMD_ReadULPage5)));}
unsigned char Uart_RATSDesFire() {return(Uart_Send(UCMD_RATSDesFire,sizeof(UCMD_RATSDesFire)));}
unsigned char Uart_DesFireGetVersion() {return(Uart_Send(UCMD_DesFireGetVersion,sizeof(UCMD_DesFireGetVersion)));}
unsigned char Uart_GetAdditionalData() {return(Uart_Send(UCMD_GetAdditionalData,sizeof(UCMD_GetAdditionalData)));}
unsigned char Uart_LightOff() {return(Uart_Send(UCMD_LightOff,sizeof(UCMD_LightOff)));}
unsigned char Uart_GetCardInformation() {return(Uart_Send(UCMD_GetCardInformation,sizeof(UCMD_GetCardInformation)));}
unsigned char Uart_GetBlcok0SecurityStatus() {return(Uart_Send(UCMD_GetBlcok0SecurityStatus,sizeof(UCMD_GetBlcok0SecurityStatus)));}
unsigned char Uart_ReadBlocks() {return(Uart_Send(UCMD_ReadBlocks,sizeof(UCMD_ReadBlocks)));}
unsigned char Uart_WriteBlock0() {return(Uart_Send(UCMD_WriteBlock0,sizeof(UCMD_WriteBlock0)));}
unsigned char Uart_WriteAFI() {return(Uart_Send(UCMD_WriteAFI,sizeof(UCMD_WriteAFI)));}
unsigned char Uart_WriteDSFID() {return(Uart_Send(UCMD_WriteDSFID,sizeof(UCMD_WriteDSFID)));}
unsigned char Uart_LockBlock0() {return(Uart_Send(UCMD_LockBlock0,sizeof(UCMD_LockBlock0)));}
unsigned char Uart_LockAFI() {return(Uart_Send(UCMD_LockAFI,sizeof(UCMD_LockAFI)));}
unsigned char Uart_LockDSFID() {return(Uart_Send(UCMD_LockDSFID,sizeof(UCMD_LockDSFID)));}

void Test_SL025x(void)
{
  #define RCVCMD g_cRxBuf[2]
  #define RCVSTA g_cRxBuf[3]
  g_cCardType=0xff;
  if(CARDIN) return;
  if((Uart_SelectCard()==0)||(RCVCMD!=0x01)||(RCVSTA!=0)) return;
  if(g_cRxBuf[g_cRxBuf[1]]==1) g_cCardType=CARDTYPE_S50;          //Mifare 1k 4 byte UID
  else if(g_cRxBuf[g_cRxBuf[1]]==2) g_cCardType=CARDTYPE_S50;     //Mifare 1k 7 byte UID
  else if(g_cRxBuf[g_cRxBuf[1]]==3) g_cCardType=CARDTYPE_UL;      //Ultralight 7 byte UID
  else if(g_cRxBuf[g_cRxBuf[1]]==4) g_cCardType=CARDTYPE_S70;     //Mifare 4k 4 byte UID
  else if(g_cRxBuf[g_cRxBuf[1]]==5) g_cCardType=CARDTYPE_S70;     //Mifare 4k 7 byte UID
  else if(g_cRxBuf[g_cRxBuf[1]]==6) g_cCardType=CARDTYPE_DES;     //Desfire 7 byte UID
  switch(g_cCardType)
  {
    case CARDTYPE_S50:
    case CARDTYPE_S70:
      //Verify password of sector0   
      if((Uart_LoginSector0()==0)||(RCVCMD!=0x02)||(RCVSTA!=0x02)) return;
      //Write data to block1			
      if((Uart_WriteBlock1()==0)||(RCVCMD!=0x04)||(RCVSTA!=0)) return;
      if(memcmp(&UCMD_WriteBlock1[4],&g_cRxBuf[4],16)!=0) return;
      //Read data from block1
      if((Uart_ReadBlock1()==0)||(RCVCMD!=0x03)||(RCVSTA!=0)) return;
      if(memcmp(&UCMD_WriteBlock1[4],&g_cRxBuf[4],16)!=0) return;
      //Verify password of sector1
      if((Uart_LoginSector1()==0)||(RCVCMD!=0x02)||(RCVSTA!=0x02)) return;
      //Initialize block4 to one purse, and value = 0x01000000
      if((Uart_WriteValue()==0)||(RCVCMD!=0x06)||(RCVSTA!=0)) return;
      if(memcmp(&UCMD_WriteValue[4],&g_cRxBuf[4],4)!=0) return;
      //Increment
      if((Uart_IncrementValue()==0)||(RCVCMD!=0x08)||(RCVSTA!=0)) return;
      //Decrement
      if((Uart_DecrementValue()==0)||(RCVCMD!=0x09)||(RCVSTA!=0)) return;
      //Backup purse to blcok5
      if((Uart_CopyValue()==0)||(RCVCMD!=0x0a)||(RCVSTA!=0)) return;
      //Read purse value from blcok5			   
      if((Uart_ReadValue()==0)||(RCVCMD!=0x05)||(RCVSTA!=0)) return;
      //Check value	 
      lPurseValue=g_cRxBuf[7];
      lPurseValue=(lPurseValue<<8)+g_cRxBuf[6];
      lPurseValue=(lPurseValue<<8)+g_cRxBuf[5];
      lPurseValue=(lPurseValue<<8)+g_cRxBuf[4];
      if(lPurseValue!=0x01000000+0x20000000-0x00000003) return;
      //Glare Red_Led to indicate working ok
      if((Uart_LightOn()==0)||(RCVCMD!=0x40)||(RCVSTA!=0)) return;
      Start_Time(30);
      while(g_bOverTime==0);
      if((Uart_LightOff()==0)||(RCVCMD!=0x40)||(RCVSTA!=0)) return;
      BuzzerOn();
      break;
    case CARDTYPE_UL:
      if((Uart_WriteULPage5()==0)||(RCVCMD!=0x11)||(RCVSTA!=0)) return;
      if((Uart_ReadULPage5()==0)||(RCVCMD!=0x10)||(RCVSTA!=0)) return;
      if (memcmp(&UCMD_WriteULPage5[4],&g_cRxBuf[4],4)!=0) return;
      //Glare Red_Led to indicate working ok
      if((Uart_LightOn()==0)||(RCVCMD!=0x40)||(RCVSTA!=0)) return;
      Start_Time(30);
      while(g_bOverTime==0);
      if((Uart_LightOff()==0)||(RCVCMD!=0x40)||(RCVSTA!=0)) return;
      BuzzerOn();
      break;
    default:
      break;
  }
}

void main(void)
{
  Init_Hardware();
  g_iTimer=0;
  while(1) Test_SL025x();
}

